{==============================================================================|
| Project : Delphree - WebProvider                               | 001.000.000 |
|==============================================================================|
| Content: WebProvider HTML Page code generators                               |
|==============================================================================|
| The contents of this file are subject to the Mozilla Public License Ver. 1.0 |
| (the "License"); you may not use this file except in compliance with the     |
| License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ |
|                                                                              |
| Software distributed under the License is distributed on an "AS IS" basis,   |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for |
| the specific language governing rights and limitations under the License.    |
|==============================================================================|
| The Original Code is WebProvider for Delphi Library.                         |
|==============================================================================|
| WebProvider HTML Page code generator developed                               |
|                                           by Istenik Petr (Czech Republic)   |
| WebProvider HTML Page code generator is Copyright (C) 1998, 1999.            |
| All Rights Reserved.                                                         |
|==============================================================================|
| Contributor(s):                                                              |
|==============================================================================|
| History: 10.2.1999 Version 001.000.000                                        |
|          First public release.                                               |
|==============================================================================}
unit WPPage;

interface

uses
	Classes, SysUtils, DB;

type
{******************************************************************************}
{ TWPContent }
{******************************************************************************}
  TWPContent = class(TComponent)
  public
    function Content: String; virtual;
    function ContentFromStream(AStream: TStream): String; virtual;
    function ContentFromString(AString: String): String; virtual;
  published
  end;

{******************************************************************************}
{ TWPPage }
{******************************************************************************}
  TTagType = (tgOther, tgLink, tgImage);
  TTagEvent = procedure(Sender: TObject; TagType: TTagType; TagString: String; TagParams: TStrings; var ReplaceText: String) of Object;

  TWPPage = class(TWPContent)
  private
    FFile: TFileName;
    FPage: TStrings;
    FOnTag: TTagEvent;
    procedure SetFile(AFile: TFileName);
    procedure SetPage(APage: TStrings);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Content: String; override;
    function ContentFromStream(AStream: TStream): String; override;
    function ContentFromString(AString: String): String; override;
  protected
    procedure DoTagEvent(TagType: TTagType; TagString: String; TagParams: TStrings; var ReplaceText: String); virtual;
  published
    property HTMLPage: TStrings
    	read FPage
      write SetPage;
    property HTMLFile: TFileName
    	read FFile
      write SetFile;
    property OnTag: TTagEvent
    	read FOnTag
      write FOnTag;
  end;

{******************************************************************************}
{ TWPDataSetPage}
{******************************************************************************}
  TWPDataSetPage = class(TWPPage)
  private
    FDataSet: TDataSet;
    function GetDataSet: TDataSet;
    procedure SetDataSet(ADataSet: TDataSet);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DoTagEvent(TagType: TTagType; TagString: String; TagParams: TStrings; var ReplaceText: String); override;
  public
    function Content: String; override;
  published
    property DataSet: TDataSet read GetDataSet write SetDataSet;
    property OnTag;
  end;

{******************************************************************************}
{ TWPTagParser}
{******************************************************************************}
  TWPTagParser = class(TObject)
  private
    FInput: TStream;
    FOutput: TStream;
    FTag: String;
    FEof: Boolean;
    FTagString: String;
    FTagParams: TStrings;
    function SeekTo(AChar: Char): String;
    procedure CopyToOutput(AString: String);
    procedure ExtractTag;
    procedure SetTag(ATag: String);
  public
    constructor Create(InputStream, OutputStream: TStream);
    destructor Destroy; override;
    procedure SkipToNextTag;
    procedure CopyTagToOutput;
  published
    property Tag: String
    	read FTag
      write SetTag;
    property TagString: String
    	read FTagString;
    property TagParams: TStrings
    	read FTagParams;
    property EOF: Boolean
    	read FEof;
  end;

procedure Register;

implementation

procedure Register;
begin
	RegisterComponents('WebProvider', [TWPPage,TWPDataSetPage]);
end;

{******************************************************************************}
{ TWPContent }
{******************************************************************************}

function TWPContent.Content: String;
begin
	Result:='';
end;

function TWPContent.ContentFromStream(AStream: TStream): String;
begin
	Result:=Content;
end;

function TWPContent.ContentFromString(AString: String): String;
begin
	Result:=Content;
end;

{******************************************************************************}
{ TWPPage }
{******************************************************************************}

constructor TWPPage.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	FPage := TStringList.Create;
end;

destructor TWPPage.Destroy;
begin
	FPage.Free;
	inherited Destroy;
end;

function TWPPage.Content: String;
var
	PageInput: TStream;
begin
	PageInput:=nil;
	try
		Result := '';
		if FFile <> '' then
			PageInput := TFileStream.Create(FFile, fmOpenRead + fmShareDenyWrite)
		else
	   	PageInput := TStringStream.Create(FPage.Text);
		if PageInput <> nil then
			Result := ContentFromStream(PageInput);
   finally
		PageInput.Free;
	end;
end;

function TWPPage.ContentFromString(AString: String): String;
var
	PageInput: TStream;
begin
	PageInput:=nil;
	try
		PageInput := TStringStream.Create(AString);
		Result := ContentFromStream(PageInput);
	finally
		PageInput.Free;
	end;
end;

function TWPPage.ContentFromStream(AStream: TStream): String;
var
	PageOutput: TStringStream;
   Parser: TWPTagParser;
   ReplaceText: String;
   TagType: TTagType;
begin
	Result:='';
	PageOutput:=nil;
	Parser:=nil;
	try
		PageOutput := TStringStream.Create('');
		Parser := TWPTagParser.Create(AStream, PageOutput);
		while not(Parser.Eof) do
		begin
      	Parser.SkipToNextTag;
         if Parser.Tag<>'' then
	         begin
					ReplaceText:=Parser.Tag;
					TagType:=tgOther;
					if Parser.TagString='LINK' then
               	TagType:=tgLink;
					if Parser.TagString='IMAGE' then
               	TagType:=tgImage;

					DoTagEvent(TagType, Parser.TagString, Parser.TagParams, ReplaceText);
               if ReplaceText<>Parser.Tag then
               	Parser.Tag:=ReplaceText;
            end;
      end;
		Result := PageOutput.DataString;
	finally
   	Parser.Free;
		PageOutput.Free;
	end;
end;

procedure TWPPage.DoTagEvent(TagType: TTagType; TagString: String; TagParams: TStrings; var ReplaceText: String);
begin
	if Assigned(FOnTag) then
		FOnTag(Self, TagType, TagString, TagParams, ReplaceText);
end;

procedure TWPPage.SetFile(AFile: TFileName);
begin
	if FFile <> AFile then
	begin
		FPage.Clear;
		FFile := AFile;
	end;
end;

procedure TWPPage.SetPage(APage: TStrings);
begin
	FPage.Assign(APage);
	FFile := '';
end;

{******************************************************************************}
{ TWPDataSetPage}
{******************************************************************************}
function TWPDataSetPage.GetDataSet: TDataSet;
begin
	Result:=FDataSet;
end;

procedure TWPDataSetPage.SetDataSet(ADataSet: TDataSet);
begin
	if FDataSet<>ADataSet then
	begin
		if ADataSet<>nil then ADataSet.FreeNotification(Self);
		FDataSet:=ADataSet;
	end;
end;

procedure TWPDataSetPage.Notification(AComponent: TComponent; Operation: TOperation);
begin
	inherited Notification(AComponent, Operation);
	if (Operation=opRemove) and (AComponent=FDataSet) then
		FDataSet:=nil;
end;

procedure TWPDataSetPage.DoTagEvent(TagType: TTagType; TagString: String; TagParams: TStrings; var ReplaceText: String);
var
	Field: TField;
begin
	if (TagParams.Count=0) and Assigned(FDataSet) then
	begin
		Field:=FDataSet.FindField(TagString);
		if Assigned(Field) then
			ReplaceText:=Field.DisplayText;
	end;
	inherited DoTagEvent(TagType, TagString, TagParams, ReplaceText);
end;

function TWPDataSetPage.Content: String;
begin
	if (FDataSet<>nil) and not FDataSet.Active then
		FDataSet.Open;
	Result:=inherited Content;
end;

{******************************************************************************}
{ TWPTagParser}
{******************************************************************************}

constructor TWPTagParser.Create(InputStream, OutputStream: TStream);
begin
	FInput := InputStream;
	FOutput := OutputStream;
	FTag:='';
   FTagString:='';
   FTagParams:=TStringList.Create;
	FEof:=False;
   FInput.Seek(0,soFromBeginning);
end;

destructor TWPTagParser.Destroy;
begin
	CopyTagToOutput;
   FTagParams.Free;
end;

function TWPTagParser.SeekTo(AChar: Char): String;
var
   RChar: Char;
begin
	Result:='';
	while (FInput.Position<FInput.Size) do
	begin
	   RChar:=#0;
   	FInput.Read(RChar,1);
		if (RChar=AChar) or (RChar=#0) then
      	Break
      else
      	Result:=Result+RChar;
	end;
   FEof:=(FInput.Position=FInput.Size);
end;

procedure TWPTagParser.SkipToNextTag;
var
	Temp: String;
begin
	if FEof then
   	Exit;
   CopyTagToOutput;
	Temp:=SeekTo(#60);
	CopyToOutput(Temp);
   Temp:=SeekTo(#62);
   if FEof then
   	Exit;
   FTag:=Temp;
   ExtractTag;
   FTag:='<'+FTag+'>';
end;

procedure TWPTagParser.ExtractTag;
var
	Par, Temp: String;
   f: Integer;
   Quoted: Boolean;
begin
	FTagString:='';
   FTagParams.Clear;
	FTagString:=LowerCase(Copy(FTag,1,Pos(' ',FTag)-1));
	FTagString:=TrimLeft(TrimRight(FTagString));
   if FTagString='' then
   begin
   	FTagString:=FTag;
      Exit;
   end;
   Temp:=Copy(FTag,Pos(' ',FTag),Length(FTag));
	Temp:=TrimLeft(TrimRight(Temp));
   if Temp='' then
   	Exit;
   Quoted := False;
	f:=1;
   repeat
      Par:='';
		while Temp[f] in [#13, #10, #32] do Inc(f);
		while True do
		begin
			while (Quoted and ((Temp[f]<>#34{"}) and (Temp[f]<>#39{'}))) or not (Temp[f] in [#0, #13, #10, #34, #39, #32]) do
         begin
         	Par:=Par+Temp[f];
         	Inc(f);
         end;
			if (Temp[f]=#34{"}) or (Temp[f]=#39{'})  then
			begin
				Quoted:=not(Quoted);
				if Quoted then
					Inc(f)
				else
         		Break;
			end
         else
         	Break;
		end;
		if Par<>'' then
      	FTagParams.Add(Par);
		if Temp[f]<>#0 then Inc(f);
	until Temp[f]=#0;
end;

procedure TWPTagParser.SetTag(ATag: String);
begin
	FTag:=ATag;
end;

procedure TWPTagParser.CopyTagToOutput;
begin
	if FTag<>'' then
   begin
   	CopyToOutput(FTag);
      FTag:='';
   end;
end;

procedure TWPTagParser.CopyToOutput(AString: String);
begin
	if FOutput <> nil then
		FOutput.WriteBuffer(AString[1], Length(AString));
end;

end.
